#!/usr/bin/env python
"""A utility for making a Google Map of image files
"""
import sys
import os
import pyexiv2
import glob

def checkGPS(filename, outfile):
    """Check for presence of GPS metadata """
    mdata=pyexiv2.ImageMetadata(filename)
    mdata.read()
    try: 
        d,m,s = mdata['Exif.GPSInfo.GPSLongitude'].value
        longitude =float(d)+float(m /60) +float(s/3600)
        longitude *= (mdata['Exif.GPSInfo.GPSLongitudeRef'].value == 'W')*-2 +1
        d,m,s = mdata['Exif.GPSInfo.GPSLatitude'].value
        latitude =float(d)+float(m /60) +float(s/3600)
        latitude *= (mdata['Exif.GPSInfo.GPSLatitudeRef'].value == 'S')*-2 +1
        title= mdata['Exif.GPSInfo.GPSDateStamp'].human_value + ' - ' + mdata['Exif.GPSInfo.GPSTimeStamp'].human_value
        placemark = '<Placemark>\n<name>'+title +'</name>\n'
        placemark += '<description> photo </description>\n'
        placemark += '<Point>\n\t<coordinates>' + str(longitude)+','+str(latitude)
        placemark += ',0</coordinates>\n</Point>\n</Placemark>'
        print filename
        print title
        print latitude,  longitude
        outfile.write(placemark)
        
    except KeyError:
        pass
        
    return  
    
def main( path):
    """"process files in list"""
    outfile = open('map.kml', 'w')
    outfile.write('<?xml version="1.0" encoding="UTF-8"?>\n<kml xmlns="http://www.opengis.net/kml/2.2">\n<Document>')

    for infile in glob.glob( os.path.join(path, '*.jpg') ):
        checkGPS(infile, outfile)
       
    outfile.write('</Document>\n</kml>')
if __name__ == "__main__":
    if len(sys.argv) > 1:
        path = sys.argv[1]
    else:
        path = './'
    main(path)
    
 
  
    
