package com.example.gpstutorial;

import java.io.File;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class GPSPhoto extends Activity {
    private static final String TAG = "GPSPhoto";
	private static final String SAVED_FILE_URI = "fileUri";
	private static final int MEDIA_TYPE_IMAGE = 0;
	private static final int MEDIA_TYPE_VIDEO = 1;
	private static final int CAPTURE_IMAGE_ACTIVITY_REQ = 0;
	
    private static final int TWO_MINUTES = 1000 * 60 * 2;
	
	private LocationManager lm;
	private TextView locationText;
	private Uri fileUri;
	private Location currentLocation;
	private String PROVIDER;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (savedInstanceState != null) {
        	String fileString = savedInstanceState.getString(SAVED_FILE_URI);
        	if (fileString != null) { fileUri = Uri.parse(fileString); }
        } 
        setContentView(R.layout.main);
        setUpButtons();
		locationText = (TextView) findViewById(R.id.location);
		lm = (LocationManager) getSystemService(LOCATION_SERVICE);
    }
    
    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
      if (fileUri != null) { savedInstanceState.putString(SAVED_FILE_URI, fileUri.toString()); }
      super.onSaveInstanceState(savedInstanceState);
    }
    
    public void setUpButtons() {
    	Button getLocationButton = (Button) findViewById(R.id.button_getlocation);
    	Button takePhotoButton = (Button) findViewById(R.id.button_takephoto);
    	Button showMapButton = (Button) findViewById(R.id.button_showmap);
    	getLocationButton.setOnClickListener(new View.OnClickListener() {
        	 public void onClick(View view) {
        		 getCurrentLocation();
    		  }
    		});   
    	takePhotoButton.setOnClickListener(new View.OnClickListener() {
    		  public void onClick(View view) {
    			  takePhoto();
    		  }
    		});   
    	showMapButton.setOnClickListener(new View.OnClickListener() {
    		  public void onClick(View view) {
    			  showMap();
    		  }
    		});   
    }
    
    public void getCurrentLocation() {
    	LocationListener locationListener = new LocationListener() {
    	    public void onLocationChanged(Location location) {
    	    	checkAndSetLocation(location);
    	    	lm.removeUpdates(this);
    	    }

    	    public void onStatusChanged(String provider, int status, Bundle extras) {}

    	    public void onProviderEnabled(String provider) {}

    	    public void onProviderDisabled(String provider) {}
    	  };

    	if (lm.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
    		PROVIDER = LocationManager.GPS_PROVIDER;
    	} else {
    		Toast.makeText(this, "GPS not available; using network location", 
    				       Toast.LENGTH_SHORT).show();
    		PROVIDER = LocationManager.NETWORK_PROVIDER;
    	}
    	currentLocation = new Location(PROVIDER);
    	lm.requestLocationUpdates(PROVIDER, 0, 0, locationListener);
    	
    	Location lastKnownLocation = lm.getLastKnownLocation(PROVIDER);
    	if (lastKnownLocation != null) {
    		checkAndSetLocation(lastKnownLocation);
    	} else {
    		locationText.setText("Can't get last known location");
    	}
    	
    }

	public void takePhoto() {
  	  Intent i = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
  	  fileUri = getOutputMediaFileUri(MEDIA_TYPE_IMAGE);
   	  if (fileUri != null) { i.putExtra(MediaStore.EXTRA_OUTPUT, fileUri); }
   	  startActivityForResult(i, CAPTURE_IMAGE_ACTIVITY_REQ);
    }
    
    public void showMap() {
    	Intent i = new Intent(getBaseContext(), GPSPhotoMap.class);
        startActivity(i);
    }
    
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
	  if (requestCode == CAPTURE_IMAGE_ACTIVITY_REQ) {
	    if (resultCode == RESULT_OK) {
	      if (data == null) {
	        Log.v(TAG, "data is null, saving in : " + fileUri);
	        // A known bug here! The image should have saved in fileUri
	        storePhoto(fileUri);
	      } else {
	        Log.v(TAG, "data is not null, saving in : " + data.getData());
	        storePhoto(data.getData());
	      }
	    } else if (resultCode == RESULT_CANCELED) {
	      // User cancelled the operation; do nothing
	    } else {
	      Log.e(TAG, "Callout for image capture failed!");
	    }
	  }
	}

	private void checkAndSetLocation(Location location) {
			if (isBetterLocation(location, currentLocation)) {
				currentLocation = location;
			}
	    	DecimalFormat latLngFormat = new DecimalFormat("#.###");
			String lat =  Double.toString(
					Double.valueOf(latLngFormat.format(currentLocation.getLatitude())));
			String lng =  Double.toString(
					Double.valueOf(latLngFormat.format(currentLocation.getLongitude())));	
			locationText.setText("Lat: " + lat + ", long: " + lng);
			
		}

	private File getOutputMediaFile(int type) {
		File directory = new File(Environment.getExternalStoragePublicDirectory(
				Environment.DIRECTORY_PICTURES), getPackageName());
		if (!directory.exists()) {
			if (!directory.mkdirs()) {
				Log.e(TAG, "Failed to create storage directory.");
				return null;
			}
		}
	
		String timeStamp = new SimpleDateFormat("yyyMMdd_HHmmss").format(new Date());
		File file;
		if (type == MEDIA_TYPE_IMAGE) {
			file = new File(directory.getPath() + File.separator + "IMG_"
					+ timeStamp + ".jpg");
		} else if (type == MEDIA_TYPE_VIDEO) {
			file = new File(directory.getPath() + File.separator + "VID_"
					+ timeStamp + ".mp4");
		} else {
			return null;
		}
		return file;
	}

	private Uri getOutputMediaFileUri(int type) {
    	File file = getOutputMediaFile(type);
    	if (file != null) { 
    	Log.v(TAG, "File is : " + file.toString());
    		return Uri.fromFile(file); 
    	} else {
    		Log.v(TAG, "File is : null" );
    		return null;
    	}
    }

    /** Determines whether one Location reading is better than the current Location fix
      * @param newLocation  The new Location that you want to evaluate
      * @param oldLocation  The current Location fix, to which you want to compare the new one
      */
    private boolean isBetterLocation(Location newLocation, Location oldLocation) {
        if (oldLocation == null) {
            return true;
        }

        long timeDiff = newLocation.getTime() - oldLocation.getTime();
        if (timeDiff > TWO_MINUTES) {
            return true;
        } else if (timeDiff < -TWO_MINUTES) {
            return false;
        }

        int accuracyDiff = (int) (newLocation.getAccuracy() - oldLocation.getAccuracy());
        if (accuracyDiff < 0 ) { return true; }
        else if (accuracyDiff > 0 ) { return false; }

        return false;
    }

    private void storePhoto(Uri uri) {
		getCurrentLocation();
		GPSPhotoDatabaseHelper dbHelper = new GPSPhotoDatabaseHelper(getBaseContext());
		SQLiteDatabase db = dbHelper.getWritableDatabase(); 
	    ContentValues values = new ContentValues();
	    values.put(GPSPhotoDatabaseHelper.KEY_FILE, uri.toString());
	    values.put(GPSPhotoDatabaseHelper.KEY_GPS_LAT, currentLocation.getLatitude());
	    values.put(GPSPhotoDatabaseHelper.KEY_GPS_LNG, currentLocation.getLongitude());
		db.insert(GPSPhotoDatabaseHelper.PHOTO_TABLE_NAME, null, values);
		db.close();
	}
    
}