package com.example.gpstutorial;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.util.ArrayList;

import android.app.Dialog;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.Drawable;
import android.view.LayoutInflater;
import android.widget.ImageView;
import android.widget.LinearLayout;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.OverlayItem;

public class PhotoLocationOverlay extends ItemizedOverlay<OverlayItem> {
	
	private ArrayList<OverlayItem> photos = new ArrayList<OverlayItem>();
	  private Context c;

	  public PhotoLocationOverlay(Drawable marker) {
	    super(boundCenterBottom(marker));
	    populate();
	  } 
	  
	  public PhotoLocationOverlay(Drawable marker, Context context) {
		  super(boundCenterBottom(marker));
		  c = context;
		  populate();
	}

	  @Override
	  public int size() {
	    return photos.size();
	  }

	  public void addItem(GeoPoint p, String photoTitle, String photoLocation) {
		OverlayItem photoItem = new OverlayItem(p, photoTitle, photoLocation);
	    photos.add(photoItem);
	    populate();
	  }
	  
	  
	  @Override
	  protected boolean onTap(int index) {
	    OverlayItem item = photos.get(index);
	    Dialog dialog = new Dialog(c);
	    dialog.setCanceledOnTouchOutside(true);
	    
	    LayoutInflater inflater = (LayoutInflater) c.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	    LinearLayout photoAlert = (LinearLayout) inflater.inflate(R.layout.photoalert, null);
	    ImageView photoImage = (ImageView) photoAlert.findViewById(R.id.photo);
        
		try {
			InputStream in = new java.net.URL(item.getSnippet()).openStream();
	        Bitmap bm = BitmapFactory.decodeStream(new FlushedInputStream(in));
            photoImage.setImageBitmap(bm);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	    
	    dialog.setContentView(photoAlert);
	    dialog.show();
	    return true;
	  }
	  
	  @Override
	  protected OverlayItem createItem(int i) {
	      return photos.get(i);
	  }

	  
	  // Deals with a known bug with decoding JPGs
	  // with thanks to the discussion at http://code.google.com/p/android/issues/detail?id=6066
	  static class FlushedInputStream extends FilterInputStream {
		    public FlushedInputStream(InputStream inputStream) {
		        super(inputStream);
		    }

		    @Override
		    public long skip(long n) throws IOException {
		        long totalBytesSkipped = 0L;
		        while (totalBytesSkipped < n) {
		            long bytesSkipped = in.skip(n - totalBytesSkipped);
		            if (bytesSkipped == 0L) {
		                  int b = read();
		                  if (b < 0) {
		                      break;  // we reached EOF
		                  } else {
		                      bytesSkipped = 1; // we read one byte
		                  }
		           }
		            totalBytesSkipped += bytesSkipped;
		        }
		        return totalBytesSkipped;
		    }
		}
}
